@@ -634,8 +634,7 @@ C
                   lwj=slw(jwind)
                   wj=sw(jwind)
                   IF(wj.lt.0.d0.or.wj.gt.1.d0) THEN
-                     call dblepr("sw",2,sw,dsw*dsw)
-                     stop
+                     wj=max(0.d0,min(1.d0,wj))
                   END IF
                   if(lwj.le.0.d0.and.wj.le.0.d0) CYCLE
                   IF(dp1.gt.1) THEN
@@ -1245,7 +1244,6 @@ C    if info>0 just keep the old estimate
             DO k=1,dp2 
                call dblepr("bi(i,k)",7,bi(i,k),1)
             END DO
-            STOP
             CYCLE
          END IF  
          DO j=1,dp1
@@ -1291,4 +1289,4 @@ C  bi(i,1) should contain either 1 or 1/sigma^2 in this case
          END DO
       END IF
       RETURN
-      END
\ No newline at end of file
+      END
